/*
 *
 *  Copyright © 2017-2018 European Support Limited
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  * Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * /
 *
 */

package org.openecomp.sdc.common.session.impl;

import org.mockito.InjectMocks;
import org.mockito.MockitoAnnotations;
import org.openecomp.sdc.common.errors.CoreException;
import org.openecomp.sdc.common.session.SessionContext;
import org.testng.Assert;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

public class AsdcSessionContextProviderTest {

    private static final String USER_ID = "cs0008";

    @InjectMocks
    private AsdcSessionContextProvider asdcSessionContextProvider;

    @BeforeMethod
    public void setUp() {
        MockitoAnnotations.initMocks(this);
    }

    @Test(expectedExceptions = CoreException.class)
    public void testGetUserIdNull() {
        asdcSessionContextProvider.create(null, null);
        asdcSessionContextProvider.get();
    }

    @Test(expectedExceptions = CoreException.class)
    public void testGetTenantNull() {
        asdcSessionContextProvider.create(USER_ID, null);
        asdcSessionContextProvider.get();
    }

    @Test
    public void testGet() {
        asdcSessionContextProvider.create(USER_ID, "tenant");
        SessionContext sessionContext = asdcSessionContextProvider.get();

        Assert.assertNotNull(sessionContext);
        Assert.assertSame(USER_ID, sessionContext.getUser().getUserId());
        Assert.assertSame("tenant", sessionContext.getTenant());
    }
}
